@@ -32,7 +32,7 @@ __declspec(thread) void *PL_current_context = NULL;
 void
 Perl_set_context(void *t)
 {
-#if defined(USE_5005THREADS) || defined(USE_ITHREADS)
+#if defined(USE_ITHREADS)
 #  ifdef USE_DECLSPEC_THREAD
     Perl_current_context = t;
 #  else
@@ -45,7 +45,7 @@ Perl_set_context(void *t)
 void *
 Perl_get_context(void)
 {
-#if defined(USE_5005THREADS) || defined(USE_ITHREADS)
+#if defined(USE_ITHREADS)
 #  ifdef USE_DECLSPEC_THREAD
     return Perl_current_context;
 #  else
@@ -61,7 +61,7 @@ Perl_get_context(void)
 BOOL
 Remove_Thread_Ctx(void)
 {
-#if defined(USE_5005THREADS) || defined(USE_ITHREADS)
+#if defined(USE_ITHREADS)
 #  ifdef USE_DECLSPEC_THREAD
 	return TRUE;
 #  else
