@@ -81,7 +81,7 @@ sub casetest {
 	if (ord('A') == 193 && $i eq "\x8A\x73") {
 	    $w = '0178'; # It's a Latin small Y with diaeresis and not a Latin small letter sharp 's'.
 	}
-	my $u = unpack "U0U", $i;
+	my $u = unpack "C0U", $i;
 	my $h = sprintf "%04X", $u;
 	my $c = chr($u); $c .= chr(0x100); chop $c;
 	foreach my $func (@funcs) {
